import {
  Button,
  Card,
  Col,
  Popover,
  Result,
  Row,
  Spin,
  Typography,
} from "antd";
import { useEffect, useState } from "react";
import { useNavigate } from "react-router-dom";
import CreateEquipmentDocumentationModal from "./CreateEquipmentDocumentationModal";
import { AppStyle, Constants, FormatDatetime, myFetch } from "../../utils";
import {
  BookOutlined,
  InfoCircleOutlined,
  ToolOutlined,
} from "@ant-design/icons";
import EditEquipmentDocumentationModal from "./EditEquipmentDocumentationModal";
import { NoteComponent } from ".";

export default function ViewEquipmentDocumentations({ scannerResult }) {
  const [equipmentDocumentationResponse, setEquipmentDocumentationResponse] =
    useState(null);
  const [isEquipmentDocumentationLoading, setIsEquipmentDocumentationLoading] =
    useState(false);

  const fetchDocumentation = () => {
    setIsEquipmentDocumentationLoading(true);

    myFetch(`/equipment/documentations/${scannerResult}`, "GET").then(
      (data) => {
        console.log("data", data);

        const updatedData = { ...data };

        // sort by date
        // last created will be on top of the list
        updatedData.Documentations = data.Documentations.sort(
          (a, b) => new Date(b.CreatedAt) - new Date(a.CreatedAt)
        );

        setIsEquipmentDocumentationLoading(false);
        setEquipmentDocumentationResponse(updatedData);
      }
    );
  };

  useEffect(() => {
    console.log("scannerResult", scannerResult);

    fetchDocumentation();
  }, [scannerResult]);

  const CreateDocumentationButton = () => {
    const InvexStockItemThumbnail = ({ width }) => {
      return (
        <img
          src={`${Constants.API_ADDRESS}/equipment/thumbnail/${scannerResult}`}
          width={width}
          style={{ borderRadius: 6 }}
        />
      );
    };

    return (
      <Row style={{ alignItems: "center" }}>
        <Col
          xs={24}
          sm={{ span: 8 }}
          md={{ span: 6 }}
          style={{
            display: "flex",
            flexDirection: "row",
            gap: 10,
            alignItems: "center",
          }}
        >
          {equipmentDocumentationResponse.Documentations.length !== 0 && (
            <Popover
              placement="right"
              trigger="hover"
              content={<InvexStockItemThumbnail width={256} />}
            >
              <>
                <InvexStockItemThumbnail width={64} />
              </>
            </Popover>
          )}

          <Typography.Title level={4} style={{ margin: 0 }}>
            {scannerResult}
          </Typography.Title>
        </Col>

        <Col xs={24} sm={{ span: 8, offset: 8 }} md={{ span: 6, offset: 12 }}>
          <CreateEquipmentDocumentationModal
            stockItemId={scannerResult}
            fetchDocumentation={fetchDocumentation}
            buttonBlock={true}
          />
        </Col>
      </Row>
    );
  };

  const DocumentationContent = ({ documentation }) => {
    return (
      <Card style={{ marginTop: AppStyle.app.margin }}>
        <div
          style={{
            display: "flex",
            flexDirection: "row",
            gap: 10,
            justifyContent: "space-between",
            marginBottom: AppStyle.typography.text.marginBottom,
          }}
        >
          <div
            style={{
              display: "flex",
              flexDirection: "row",
              gap: 10,
              alignItems: "center",
            }}
          >
            <DocumentationTypeIcon type={documentation.Type} />
            <Typography.Title level={4} style={{ margin: 0 }}>
              {documentation.Title}
            </Typography.Title>
          </div>

          <div
            style={{
              display: "flex",
              flexDirection: "row",
              gap: 10,
              alignItems: "center",
            }}
          >
            <Popover
              title={
                <Typography.Title
                  level={4}
                  style={{ color: Constants.COLORS.SECONDARY }}
                >
                  Details
                </Typography.Title>
              }
              trigger="click"
              placement="left"
              content={
                <p style={{ color: "#000", margin: 0 }}>
                  <b>ID:</b> {documentation.Id}
                  <br />
                  <b>Type:</b> {documentation.Type}
                  <br />
                  <b>Created at:</b> {FormatDatetime(documentation.CreatedAt)}
                  <br />
                  <b>Updated at:</b> {FormatDatetime(documentation.UpdatedAt)}
                </p>
              }
            >
              <InfoCircleOutlined
                style={{ fontSize: 24, color: Constants.COLORS.ICON_INFO }}
                onClick={() => console.log("info")}
              />
            </Popover>

            <EditEquipmentDocumentationModal
              stockItemId={scannerResult}
              documentationId={documentation.Id}
              fetchDocumentation={fetchDocumentation}
            />
          </div>
        </div>

        {documentation.Notes !== "" &&
          JSON.parse(documentation.Notes).map((note, index) => {
            console.log("map doc");

            return (
              <NoteComponent
                key={index}
                viewMode
                image={note.Image}
                documentationId={documentation.Id}
                description={note.Description}
              />
            );
          })}
      </Card>
    );
  };

  if (isEquipmentDocumentationLoading) {
    return (
      <div
        style={{
          display: "flex",
          alignItems: "center",
          justifyContent: "center",
          marginTop: 120,
        }}
      >
        <Spin size="large" />
      </div>
    );
  }

  if (equipmentDocumentationResponse === null) return null;

  // backend unauthorized to access invex
  if (equipmentDocumentationResponse.Status === 401) {
    return (
      <Result
        status="403"
        title="401"
        subTitle="The backend server is not authorized to access invex."
      />
    );
  }

  // stock item doesn't exists on invex
  if (equipmentDocumentationResponse.Status === 404) {
    return (
      <Result
        status="500"
        title="500"
        subTitle="The scanned item doesn't exists on invex"
      />
    );
  }

  if (equipmentDocumentationResponse.Documentations.length === 0) {
    return (
      <Result
        status="404"
        title="404"
        subTitle="Sorry, for the equipment does not exist an documentation."
        extra={[
          <CreateEquipmentDocumentationModal
            key="0"
            stockItemId={scannerResult}
            fetchDocumentation={fetchDocumentation}
            buttonBlock={false}
          />,
        ]}
      />
    );
  }

  return (
    <>
      <CreateDocumentationButton />

      {equipmentDocumentationResponse.Documentations.map(
        (documentation, index) => (
          <DocumentationContent key={index} documentation={documentation} />
        )
      )}
    </>
  );
}

function DocumentationTypeIcon({ type }) {
  switch (type) {
    case 1:
      return <ToolOutlined style={{ fontSize: 24 }} />;
    case 2:
      return <BookOutlined style={{ fontSize: 24 }} />;
    default:
      console.log("DocumentationTypeIcon type not found:", type);
      return null;
  }
}
