import {
  ArrowDownOutlined,
  ArrowUpOutlined,
  CameraOutlined,
  DeleteOutlined,
  EllipsisOutlined,
  FullscreenOutlined,
  PlusOutlined,
  SearchOutlined,
} from "@ant-design/icons";
import {
  Button,
  Card,
  Col,
  Divider,
  Dropdown,
  Input,
  Result,
  Row,
  Select,
  Space,
  Typography,
  Upload,
  message,
} from "antd";
import { AppStyle, Constants, myFetch } from "../../utils";
import { useEffect, useRef, useState } from "react";
import { QrScanner } from "@yudiel/react-qr-scanner";
import ViewEquipmentDocumentations from "./ViewEquipmentDocumentation";
import MyModal, {
  MyModalCloseCreateButtonFooter,
  MyModalCloseSaveButtonFooter,
} from "../../Components/MyModal";
import Webcam from "react-webcam";
import TextArea from "antd/es/input/TextArea";

message.config({
  maxCount: 2,
});

export default function EquipmentDocumentationOverview() {
  const [scannerResult, setScannerResult] = useState("");
  const [isScannerActive, setIsScannerActive] = useState(false);
  const [inputValue, setInputValue] = useState("");

  console.log("scan", scannerResult);

  const isScannedQrCodeValid = (result) => {
    // {"stockitem": 11} or 11 is valid

    try {
      // Try to parse the scanned content as JSON
      const parsedResult = JSON.parse(result);

      // Check if the parsed object has the expected format
      if (parsedResult && typeof parsedResult === "object") {
        if (
          parsedResult.stockitem !== undefined &&
          typeof parsedResult.stockitem === "number"
        ) {
          return true; // The QR code has the expected format
        }
      } else if (!isNaN(parsedResult)) {
        return true; // The QR code is a valid number
      }

      return false; // The QR code does not have the expected format or value
    } catch (error) {
      return false; // Error while parsing the QR code
    }
  };

  return (
    <>
      <Row style={{ marginBottom: AppStyle.app.margin }}>
        <Col xs={0} md={7} />
        <Col xs={24} md={10}>
          <Card
            style={{
              textAlign: "center",
            }}
          >
            {isScannerActive ? (
              <>
                <QrScanner
                  onDecode={(result) => {
                    console.log(result);

                    if (!isScannedQrCodeValid(result)) {
                      message.error("Invalid stock item QR code");
                      return;
                    }

                    setScannerResult(JSON.parse(result).stockitem.toString());
                    setIsScannerActive(false);
                  }}
                  onError={(error) => console.log(error?.message)}
                />
                <Button
                  block
                  onClick={() => setIsScannerActive(false)}
                  style={{
                    marginTop: AppStyle.app.margin,
                    marginBottom: AppStyle.app.margin,
                  }}
                >
                  Close camera
                </Button>
              </>
            ) : (
              <div onClick={() => setIsScannerActive(true)}>
                <CameraOutlined style={{ fontSize: 64 }} />
                <Typography.Title level={5}>Scan equipment</Typography.Title>
              </div>
            )}

            <Row gutter={AppStyle.grid.row.gutter}>
              <Col xs={24} xl={16}>
                <Input
                  placeholder="Equipment id"
                  value={inputValue}
                  onInput={(e) => setInputValue(e.target.value)}
                />
              </Col>
              <Col xs={24} xl={8}>
                <Button
                  block
                  icon={<SearchOutlined />}
                  onClick={() => {
                    if (!isScannedQrCodeValid(inputValue)) {
                      message.error("Invalid stock item code");
                      return;
                    }

                    setScannerResult(inputValue);
                  }}
                >
                  Search
                </Button>
              </Col>
            </Row>
          </Card>
        </Col>
        <Col xs={0} md={7} />
      </Row>

      {scannerResult === "" ? (
        <Result
          status="404"
          title="No equipment scanned"
          subTitle="Scan a equipment to see the documentation."
        />
      ) : (
        <ViewEquipmentDocumentations scannerResult={scannerResult} />
      )}
    </>
  );
}

export const EmptyNote = { Image: null, Description: "" };

const selectDocumentationTypeOptions = [
  { value: 1, label: "Repair protocol" },
  { value: 2, label: "Documentation" },
];

export function EquipmentDocumentationViewEditComponent({
  createMode,
  isOpen,
  onCancel,
  fetchDocumentation,
  stockItemId,
  documentationId,
}) {
  const [title, setTitle] = useState(createMode ? "New documentation" : "");
  const [selectedDocumentationType, setSelectedDocumentationType] = useState(
    selectDocumentationTypeOptions[0].value
  );
  const [notes, setNotes] = useState([EmptyNote]);
  const [isDocumentationUploading, setIsDocumentationUploading] =
    useState(false);
  const documentationResponse = useRef();

  const handleCreate = () => {
    setIsDocumentationUploading(true);

    const updatedNotes = [...notes];

    updatedNotes.forEach((note, index) => {
      if (note.Image === null && note.Description === "") {
        updatedNotes.splice(index, 1);
      }
    });

    let body = {
      stockItemId: stockItemId,
      type: selectedDocumentationType,
      title: title,
      notes: updatedNotes,
    };

    console.log("body", body);

    myFetch(`/equipment/documentation/create`, "POST", body, {}).then(
      (data) => {
        console.log("data", data);

        setIsDocumentationUploading(false);
        fetchDocumentation();
        onCancel();
      }
    );
  };

  const handleEdit = () => {
    setIsDocumentationUploading(true);

    const updatedNotes = [...notes];

    console.log("documentationResponse.current", documentationResponse.current);

    updatedNotes.forEach((note, index) => {
      if (note.Image?.startsWith("http")) {
        updatedNotes[index].Image = JSON.parse(
          documentationResponse.current.Notes
        )[index].Image;
      }
    });

    let body = {
      documentationId: documentationId,
      type: selectedDocumentationType,
      title: title,
      notes: updatedNotes,
    };

    console.log("body", body);

    myFetch(`/equipment/documentation/edit`, "POST", body, {}).then((data) => {
      console.log("data", data);

      setIsDocumentationUploading(false);
      fetchDocumentation();
      onCancel();
    });
  };

  const handleDescriptionChange = (index) => (e) => {
    const updatedNotes = [...notes];

    updatedNotes[index] = {
      ...updatedNotes[index],
      Description: e.target.value,
    };

    setNotes(updatedNotes);
  };

  const handleImageChange = (index) => (newImage) => {
    const updatedNotes = [...notes];

    updatedNotes[index] = {
      ...updatedNotes[index],
      Image: newImage,
    };

    setNotes(updatedNotes);
  };

  const handleAddNote = () => setNotes([...notes, EmptyNote]);

  const isAddNoteButtonDisabled = () => {
    const lastNote = notes[notes.length - 1];

    return lastNote.Image === null && lastNote.Description === "";
  };

  const isCreateButtonDisabled = () => {
    if (notes.length === 0) return true;

    if (notes.length === 1) {
      return notes[0].Image === null && notes[0].Description === "";
    }

    return false;
  };

  // triggers on edit documentation
  useEffect(() => {
    if (!isOpen) return;
    if (createMode) return;

    myFetch(
      `/equipment/documentation/${stockItemId}/${documentationId}`,
      "GET"
    ).then((data) => {
      console.log("data", data);

      documentationResponse.current = data;

      setTitle(data.Title);

      // adding the static server url in front of the file name
      const updatedDataNotes = JSON.parse(data.Notes);

      updatedDataNotes.forEach((note, index) => {
        if (note.Image !== "") {
          updatedDataNotes[
            index
          ].Image = `${Constants.STATIC_CONTENT_ADDRESS}equipmentdocumentation/${documentationId}/${note.Image}`;
        }
      });

      setNotes(updatedDataNotes);
      setSelectedDocumentationType(data.Type);
    });
  }, [isOpen]);

  return (
    <MyModal
      isOpen={isOpen}
      onCancel={onCancel}
      footer={
        createMode ? (
          <MyModalCloseCreateButtonFooter
            onCreate={handleCreate}
            onCancel={onCancel}
            isCreateButtonDisabled={isCreateButtonDisabled()}
            isCreateButtonLoading={isDocumentationUploading}
          />
        ) : (
          <MyModalCloseSaveButtonFooter
            onSave={handleEdit}
            onCancel={onCancel}
            isSaveButtonLoading={isDocumentationUploading}
          />
        )
      }
    >
      <Typography.Title
        editable={{ text: title, onChange: setTitle }}
        level={1}
      >
        {title}
      </Typography.Title>

      <div style={{ marginBottom: AppStyle.typography.text.marginBottom }}>
        <Typography.Text>Documentation type</Typography.Text>
      </div>

      <Select
        value={selectedDocumentationType}
        style={{ width: "100%", marginBottom: AppStyle.app.margin }}
        onChange={(value) => setSelectedDocumentationType(value)}
        options={selectDocumentationTypeOptions}
      />

      {notes.map((note, index) => (
        <NoteComponent
          key={index}
          index={index}
          image={note.Image}
          onImageChange={handleImageChange(index)}
          description={note.Description}
          onDescriptionChange={handleDescriptionChange(index)}
          onDeleteImage={() => handleImageChange(index)(null)}
          documentationId={documentationId}
          onMoveUp={() => {
            if (index === 0) return;

            // updating the notes
            const updatedNotes = [...notes];
            const temp = updatedNotes[index - 1];
            updatedNotes[index - 1] = updatedNotes[index];
            updatedNotes[index] = temp;

            setNotes(updatedNotes);

            if (createMode) return;

            // updating the documentationResponse.current
            const updatedCurrentNotes = JSON.parse(
              documentationResponse.current.Notes
            );

            const temp2 = updatedCurrentNotes[index - 1];
            updatedCurrentNotes[index - 1] = updatedCurrentNotes[index];
            updatedCurrentNotes[index] = temp2;

            documentationResponse.current = {
              ...documentationResponse.current,
              Notes: JSON.stringify(updatedCurrentNotes),
            };
          }}
          onMoveDown={() => {
            if (index === notes.length - 1) return;

            // updating the notes
            const updatedNotes = [...notes];
            const temp = updatedNotes[index + 1];
            updatedNotes[index + 1] = updatedNotes[index];
            updatedNotes[index] = temp;

            setNotes(updatedNotes);

            if (createMode) return;

            // updating the documentationResponse.current
            const updatedCurrentNotes = JSON.parse(
              documentationResponse.current.Notes
            );

            const temp2 = updatedCurrentNotes[index + 1];
            updatedCurrentNotes[index + 1] = updatedCurrentNotes[index];
            updatedCurrentNotes[index] = temp2;

            documentationResponse.current = {
              ...documentationResponse.current,
              Notes: JSON.stringify(updatedCurrentNotes),
            };
          }}
          onDelete={() => {
            if (notes.length === 1) {
              setNotes([EmptyNote]);
              return;
            }

            const updatedNotes = [...notes];
            updatedNotes.splice(index, 1);

            setNotes(updatedNotes);
          }}
        />
      ))}

      <div style={{ textAlign: "center" }}>
        <Button
          type="primary"
          disabled={isAddNoteButtonDisabled()}
          icon={<PlusOutlined />}
          onClick={handleAddNote}
        >
          Add note
        </Button>
      </div>
    </MyModal>
  );
}

function UploadComponent({ setImagePreview }) {
  const handleBeforeUpload = (file) => {
    if (
      !Constants.ACCEPTED_EQUIPMENT_DOCUMENTATION_FILE_TYPES.includes(file.type)
    ) {
      message.error(`${file.name} is not valid file type`);
      return false;
    }

    const reader = new FileReader();
    reader.onload = () => setImagePreview(reader.result);
    reader.readAsDataURL(file);

    return false;
  };

  return (
    <Upload
      beforeUpload={handleBeforeUpload}
      maxCount={1}
      accept={Constants.ACCEPTED_EQUIPMENT_DOCUMENTATION_FILE_TYPES}
      showUploadList={false}
    >
      <PlusOutlined />
    </Upload>
  );
}

function CameraComponent({ setImagePreview }) {
  const [modalVisible, setModalVisible] = useState(false);
  const [cameraVisible, setCameraVisible] = useState(false);
  const webcamRef = useRef(null);

  const handleCapture = () => {
    const imageSrc = webcamRef.current.getScreenshot();
    setImagePreview(imageSrc);

    handleCancel();
  };

  const handleCancel = () => {
    setCameraVisible(false);
    // this is needed to close the camera correctly
    setTimeout(() => setModalVisible(false), 100);
  };

  return (
    <>
      <div
        onClick={() => {
          setModalVisible(true);
          setCameraVisible(true);
        }}
      >
        <CameraOutlined />
      </div>

      <MyModal
        isOpen={modalVisible}
        onCancel={handleCancel}
        footer={[
          <Button key={0} block type="primary" onClick={handleCapture}>
            Take picture
          </Button>,
        ]}
      >
        {cameraVisible && (
          <Webcam
            audio={false}
            ref={webcamRef}
            screenshotFormat="image/jpeg"
            width={"100%"}
            videoConstraints={{
              width: 1920,
              height: 1080,
              //facingMode: { exact: "environment" },
            }}
          />
        )}
      </MyModal>
    </>
  );
}

function MoreComponent({ onMoveUp, onMoveDown, onDelete }) {
  return (
    <Dropdown
      trigger={["click"]}
      menu={{
        items: [
          {
            label: (
              <div onClick={onMoveUp}>
                <Space>
                  <ArrowUpOutlined />
                  <span>Move up</span>
                </Space>
              </div>
            ),
            key: "1",
          },
          {
            label: (
              <div onClick={onMoveDown}>
                <Space>
                  <ArrowDownOutlined />
                  <span>Move down</span>
                </Space>
              </div>
            ),
            key: "2",
          },
          {
            label: (
              <div onClick={onDelete}>
                <Space>
                  <DeleteOutlined />
                  <span>Delete note</span>
                </Space>
              </div>
            ),
            key: "0",
          },
        ],
      }}
    >
      <EllipsisOutlined />
    </Dropdown>
  );
}

export function NoteComponent({
  viewMode,
  image,
  onImageChange,
  description,
  onDescriptionChange,
  onDeleteImage,
  documentationId,
  onMoveUp,
  onMoveDown,
  onDelete,
}) {
  const [isImageFullScreenModalOpen, setIsImageFullScreenModalOpen] =
    useState(false);

  const imageSrc = viewMode
    ? `${Constants.STATIC_CONTENT_ADDRESS}equipmentdocumentation/${documentationId}/${image}`
    : image;

  const FullscreenOutlinedIcon = ({ disabled }) => (
    <FullscreenOutlined
      disabled={true}
      onClick={() => {
        if (disabled) return;

        setIsImageFullScreenModalOpen(true);
      }}
    />
  );

  return (
    <div style={{ marginBottom: AppStyle.app.margin }}>
      <Row gutter={[AppStyle.grid.row.gutter[0]]}>
        <Col xs={24} md={8}>
          <Card
            bodyStyle={{ padding: 0 }}
            actions={
              viewMode
                ? image
                  ? [<FullscreenOutlinedIcon />]
                  : null
                : [
                    <UploadComponent setImagePreview={onImageChange} />,
                    <CameraComponent setImagePreview={onImageChange} />,
                    <DeleteOutlined onClick={onDeleteImage} />,
                    <FullscreenOutlinedIcon
                      disabled={image === null || image === ""}
                    />,
                    <MoreComponent
                      onMoveUp={onMoveUp}
                      onMoveDown={onMoveDown}
                      onDelete={onDelete}
                    />,
                  ]
            }
          >
            {!image ? (
              <div
                style={{
                  height: 250,
                  display: "flex",
                  justifyContent: "center",
                  alignItems: "center",
                }}
              >
                {viewMode ? "No image" : "No image selected"}
              </div>
            ) : (
              <img
                src={imageSrc}
                width="100%"
                style={{
                  borderTopLeftRadius: 6,
                  borderTopRightRadius: 6,
                }}
                alt="Preview"
              />
            )}
          </Card>
        </Col>

        <Col xs={24} md={16}>
          {viewMode ? (
            <Typography.Text>{description}</Typography.Text>
          ) : (
            <TextArea
              rows={8}
              placeholder="Description"
              value={description}
              onChange={onDescriptionChange}
            />
          )}
        </Col>
      </Row>

      <ImageFullscreenModal
        isOpen={isImageFullScreenModalOpen}
        onCancel={() => setIsImageFullScreenModalOpen(false)}
        image={imageSrc}
      />
    </div>
  );
}

function ImageFullscreenModal({ isOpen, onCancel, image }) {
  return (
    <MyModal isOpen={isOpen} onCancel={onCancel}>
      <img width="100%" src={image} alt="Fullscreen preview" />
    </MyModal>
  );
}
